<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\App\State;
use Magento\Store\Api\StoreConfigManagerInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Translations
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Search extends AbstractExporter
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager
    ) {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
        $this->storeManager = $storeManager;
    }

    /**
     * @return iterable
     */
    public function getCollections(): iterable
    {
        yield 'search';
    }

    public function preExecute()
    {
        $this->getConnection()->dropCollection(
            'search',
            $this->getDatabase()
        );
    }

    /*
     * {
    "_id" : 1,
    "query_id" : 1,
    "query_text" : "Test Stefano",
    "num_results" : "0",
    "popularity" : "0",
    "redirect" : null,
    "store_id" : 1,
    "display_in_terms" : "0",
    "is_active" : 1,
    "is_processed" : "0",
    "updated_at" : "2021-06-24 08:16:28",
    "use_custom_rule" : 1,
    "query_categories" : "7,9,11,20,29,39",
    "query_tags" : [
        "f00",
        "b4r"
    ],
    "query_price" : [
        0,
        100
    ],
    "query_stock" : "0",
    "pinned_products" : "24-MB01,24-MB04,24-MB03",
    "store" : "it"
}
     */
    public function execute()
    {

        $allStores = $this->storeManager->getStores();

        $bulkData = [];
        $_id = 1;
        $query = 'SELECT * FROM search_query';

        $result = $this->resourceConnection->getConnection()->fetchAll($query);

        $func = function($value) {
            return (json_decode($value)) ? json_decode($value) : $value;
        };

        foreach ($result as $item) {

            $item = array_map($func, $item);
            $item['_id'] = $_id;
            $item['store'] = $allStores[(int)$item['store_id']]->getData('code');
            $item['pinned_products'] = $this->getProductIds($item['pinned_products']??'');
	    /// TODO: extrat categoires
            ///$item['query_categories'] = $item['query_categories'];
            $bulkData[] = $item;
            $_id++;
        }

        try {
            $this->getConnection()->update(
                '_id',
                $bulkData,
                'search',
                true,
                $this->getDatabase()
            );
        } catch (\MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @param $pinned_products
     * @return array
     */
    public function getProductIds($pinned_products)
    {
        $skus = implode(',', array_map(function($a) { return "'$a'";}, explode(',',$pinned_products?? '') ??[]) );
        return $this->resourceConnection->getConnection()->fetchCol("SELECT entity_id FROM catalog_product_entity WHERE sku IN ($skus)");
    }
}
