<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Model\Importer;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class Translation
 * @package FiloBlu\Refilo\Model\Importer
 */
class Translation
{
    /**
     * @var \Magento\Framework\Module\Dir
     */
    private $moduleDirectory;
    /**
     * @var \Magento\Framework\File\Csv
     */
    private $csv;
    /**
     * @var \Magento\Translation\Model\ResourceModel\Translate
     */
    private $resourceTranslate;
    /**
     * @var \Magento\Translation\Model\ResourceModel\StringUtils
     */
    private $stringUtils;
    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    private $curl;
    /**
     * @var ScopeConfigInterface
     */
    private $config;
    /**
     * @var \Magento\Framework\Filesystem
     */
    private $filesystem;
    /**
     * @var \FiloBlu\Refilo\Helper\Data
     */
    private $helper;

    /**
     * Translation constructor.
     * @param \Magento\Framework\Module\Dir $moduleDirectory
     * @param \Magento\Framework\File\Csv $csv
     * @param \Magento\Translation\Model\ResourceModel\Translate $resourceTranslate
     * @param \Magento\Translation\Model\ResourceModel\StringUtils $stringUtils
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\Filesystem $filesystem
     * @param ScopeConfigInterface $config
     * @param \FiloBlu\Refilo\Helper\Data $helper
     */
    public function __construct(
        \Magento\Framework\Module\Dir $moduleDirectory,
        \Magento\Framework\File\Csv $csv,
        \Magento\Translation\Model\ResourceModel\Translate $resourceTranslate,
        \Magento\Translation\Model\ResourceModel\StringUtils $stringUtils,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\Filesystem $filesystem,
        ScopeConfigInterface $config,
        \FiloBlu\Refilo\Helper\Data $helper
    )
    {
        $this->moduleDirectory = $moduleDirectory;
        $this->csv = $csv;
        $this->resourceTranslate = $resourceTranslate;
        $this->stringUtils = $stringUtils;
        $this->curl = $curl;
        $this->config = $config;
        $this->filesystem = $filesystem;
        $this->helper = $helper;
    }

    /**
     * @param string $locale
     */
    public function import(string $locale)
    {
        foreach ($this->getTranslationsToAdd($locale) as $string => $translate) {
            $this->stringUtils->saveTranslate($string, $translate, $locale, 0);
        }
    }

    /**
     * @param string $locale
     * @return array
     */
    public function getTranslationsToAdd(string $locale)
    {
        $storeFrontTranslations = $this->loadFromStoreFront();
        $requiredTranslations = array_merge($storeFrontTranslations, $this->loadFormFile($locale));
        return array_diff_key($requiredTranslations, $this->loadFromDatabase($locale));
    }

    /**
     *
     */
    public function loadFromStoreFront()
    {
        $baseUrl = $this->helper->getStoreFrontBaseUrl(0);

        if(!$baseUrl) {
            return [];
        }

        $url = trim($baseUrl, '/') . '/translations';

        $this->curl->get($url);

        // TODO: provide some form of logging
        if((int)$this->curl->getStatus() !== 200) {
            return [];
        }

        $body = $this->curl->getBody();

        /** @var \Magento\Framework\Filesystem\Directory\Write $directory */
        $directory = $this->filesystem->getDirectoryWrite(
            DirectoryList::TMP
        );

        $directory->create();

        $tmpFileName = $directory->getAbsolutePath(
            'store_front_lables' . uniqid((string)\Magento\Framework\Math\Random::getRandomNumber()) . time() . '.csv'
        );

        file_put_contents($tmpFileName, $body);

        $translations = $this->csv->getDataPairs($tmpFileName);

        array_shift($translations);

        $directory->delete($directory->getRelativePath($tmpFileName));
        return $translations;
    }

    /**
     * @param string $locale
     * @return array
     */
    public function loadFormFile(string $locale)
    {
        $root = $this->moduleDirectory->getDir('FiloBlu_Refilo');
        $translationsFile = implode(DIRECTORY_SEPARATOR, [$root, 'labels', "{$locale}.csv"]);
        return $this->csv->getDataPairs($translationsFile) ?? [];
    }

    /**
     * @param string $locale
     * @return array
     */
    public function loadFromDatabase(string $locale)
    {
        return $this->resourceTranslate->getTranslationArray(0, $locale);
    }
}
